package timezonePatch.model;

import java.util.SimpleTimeZone;

public class TimezonePatch
{
   private String id;
   private int gmt;
   private DST dst;
   private final int ONE_HOUR = 1000*60*60;

   public TimezonePatch(){}

   public TimezonePatch(
            String _id,
            int _gmt,
            DST _dst)
   {
    id = _id;
    gmt = _gmt;
    dst = _dst;
   }

   public TimezonePatch(int test)
   {
     id = "America/Sao_Paulo";
     gmt = test;
     dst = new DST("test" + test);
   }

    public String getId()
    {
        return id;
    }

    public void setId(String s)
    {
        id = s;
    }

    public int getGmt()
    {
        return gmt;
    }

    public void setGmt(int i)
    {
        gmt = i;
    }

    public DST getDst()
    {
        return dst;
    }

    public void setDst(DST o)
    {
        dst = o;
    }
    public boolean hasDst()
    {
        return dst!= null;
    }

    public boolean valid() throws InvalidTimeZonePatchtException
    {
      if (id==null)
        throw new InvalidTimeZonePatchtException("invalid id: " + id);

      if (id.equals(null))
        throw new InvalidTimeZonePatchtException("invalid id: " + id);

      if ((gmt<-12)||(gmt>12))
        throw new InvalidTimeZonePatchtException("invalid gmt: " + gmt);
      if (dst != null)
        dst.valid();
      return true;
    }

    public SimpleTimeZone toSimpleTimeZone() throws InvalidTimeZonePatchtException
    {
      this.valid();
      if (dst != null){
        Date startDate = dst.getStartDate();
        Date endDate = dst.getEndDate();
        return new SimpleTimeZone(
          // offset
          gmt,
          // timezone id
          id,
          // dst start month
          startDate.getMonth() - 1,
          // dst start day
          startDate.getDay(), 0,
          // dst start hour
          startDate.getHour() * ONE_HOUR,
          // dst end month
          endDate.getMonth() - 1,
          // dst end day
          endDate.getDay(), 0,
          // dst end hour
          endDate.getHour() * ONE_HOUR);
      }
      else {
        return new SimpleTimeZone(
            // offset
            gmt,
            // timezone id
            id);
      }
  }

    public String toString()
    {
      StringBuffer strBuff = new StringBuffer();
      strBuff.append("Timezone Patch:");
      strBuff.append(id);
      strBuff.append(" | GMT ");
      strBuff.append(gmt);
      strBuff.append(" | DST patch: ");
      strBuff.append(dst);
      return strBuff.toString();
    }


}
