package timezonePatch.test;

import junit.framework.*;
import java.util.Date;
import java.util.Calendar;
import java.util.TimeZone;

import timezonePatch.TimeZonePatch;

import java.text.SimpleDateFormat;

public class TestTimeZone extends TestCase {
    private TimeZone timeZone;
    public TestTimeZone(String name) {
        super(name);
    }

    protected void setUp() throws Exception {
        timeZone = TimeZonePatch.getTimeZone(DST.TIMEZONE_LOCATION);
//        timeZone = TimeZone.getTimeZone(DST.TIMEZONE_LOCATION);
        super.setUp();
    }

    private boolean entryDST(TimeZone timezone, String entryDate, String dateFormat) throws Exception {

        Date startDST = this.stringToDate(entryDate,dateFormat);
        //one day after DST
        boolean oneDayAfterExitDSTDate = timezone.inDaylightTime(this.dayAfter(startDST));
        // one day before DST
        boolean oneDayBeforeEntryDSTDate = timezone.inDaylightTime(this.previousDay(startDST));
        // correct values:
        // oneDayBeforeEntryDSTDate == false &&
        // oneDayAfterExitDSTDate == true
        return (!(oneDayBeforeEntryDSTDate) && (oneDayAfterExitDSTDate));
    }

    private boolean exitDST(TimeZone timezone, String exitDate, String dateFormat) throws Exception {
        Date endDST = this.stringToDate(exitDate,dateFormat);
        //one day after DST
        boolean oneDayAfterExitDSTDate = timezone.inDaylightTime(this.dayAfter(endDST));
        // one day before DST
        boolean oneDayBeforeExitDSTDate = timezone.inDaylightTime(this.previousDay(endDST));
        // correct values:
        // oneDayBeforeExitDSTDate == true &&
        // oneDayAfterExitDSTDate == false
        return ((oneDayBeforeExitDSTDate) && !(oneDayAfterExitDSTDate));
    }

    public void testValidTimeZone() throws Exception {
        boolean expectedReturn = true;

        String[] allTimeZoneIds = timeZone.getAvailableIDs();
        boolean foundTimezone = false;
        for (int i=0; i<allTimeZoneIds.length; i++)
        {
            if(DST.TIMEZONE_LOCATION.equals(allTimeZoneIds[i]))
            {
              foundTimezone = true;
              break;
            }
        }
        System.out.println(allTimeZoneIds.length);
        boolean actualReturn = true;
        assertEquals("valid timezone location ", expectedReturn, foundTimezone);
    }

    public void testEntryDST() throws Exception {
        boolean expectedReturn = true;
        boolean actualReturn = this.entryDST(timeZone,DST.START_DST,DST.DATE_FORMAT);
        assertEquals("DST enter date", expectedReturn, actualReturn);
    }

    public void testExitDST() throws Exception {
        boolean expectedReturn = true;
        boolean actualReturn = this.exitDST(timeZone,DST.END_DST,DST.DATE_FORMAT);
        assertEquals("DST exit date", expectedReturn, actualReturn);
    }

    private Date stringToDate(String dateStr, String dateFormat) throws
        java.text.ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(dateFormat);
        Date dstChangeTime = simpleDateFormat.parse(dateStr);
        Date dstChangeTime_12HoursLater = new Date(dstChangeTime.getTime()+1000 * 60 * 60 * 12);
        return dstChangeTime_12HoursLater;
    }


    private Date previousDay(Date date) {
        // Get yesterday's date.
        long time = date.getTime();
        return new Date(time - 1000 * 60 * 60 * 24);
    }

    private Date dayAfter(Date date) {
        // Get yesterday's date.
        long time = date.getTime();
        return new Date(time + 1000 * 60 * 60 * 24);
    }

}

